;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-*-mode:scheme-*-
;;;                                                                       ;;
;;;                            [SOMEBODY]                                 ;;
;;;                         Copyright (c) 2000                            ;;
;;;                        All Rights Reserved.                           ;;
;;;                                                                       ;;
;;;  Distribution policy                                                  ;;
;;;     [CHOOSE ONE OF]                                                   ;;
;;;     Free for any use                                                  ;;
;;;     Free for non commercial use                                       ;;
;;;     something else                                                    ;;
;;;                                                                       ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  An example diphone voice
;;;
;;;  Authors: [The people who did the work]
;;;

;;; Try to find out where we are
(if (assoc 'net_dutch_frans_diphone voice-locations)
    (defvar net_dutch_frans_dir 
      (cdr (assoc 'net_dutch_frans_diphone voice-locations)))
    ;;; Not installed in Festival yet so assume running in place
    (defvar net_dutch_frans_dir (pwd)))

(if (not (probe_file (path-append net_dutch_frans_dir "festvox/")))
    (begin
     (format stderr "net_dutch_frans: Can't find voice scm files they are not in\n")
     (format stderr "   %s\n" (path-append net_dutch_frans_dir "festvox/"))
     (format stderr "   Either the voice isn't linked into Festival\n")
     (format stderr "   or you are starting festival in the wrong directory\n")
     (error)))

;;;  Add the directory contains general voice stuff to load-path
(set! load-path (cons (path-append net_dutch_frans_dir "festvox/") load-path))

;;; Voice specific parameter are defined in each of the following
;;; files
(require 'net_dutch_frans_phoneset)
(require 'net_dutch_frans_tokenizer)
(require 'net_dutch_frans_tagger)
(require 'net_dutch_frans_lexicon)
(require 'net_dutch_frans_phrasing)
(require 'net_dutch_frans_intonation)
(require 'net_dutch_frans_duration)
(require 'net_dutch_frans_f0model)
(require 'net_dutch_frans_other)
;; ... and others as required

;;;  Ensure we have a festival with the right diphone support compiled in
(require_module 'UniSyn)

(set! net_dutch_frans_lpc_sep 
      (list
       '(name "net_dutch_frans_lpc_sep")
       (list 'index_file (path-append net_dutch_frans_dir "dic/fransdiph.est"))
       '(grouped "false")
       (list 'coef_dir (path-append net_dutch_frans_dir "lpc"))
       (list 'sig_dir  (path-append net_dutch_frans_dir "lpc"))
       '(coef_ext ".lpc")
       '(sig_ext ".res")
       (list 'default_diphone 
	     (string-append
	      (car (cadr (car (PhoneSet.description '(silences)))))
	      "-"
	      (car (cadr (car (PhoneSet.description '(silences)))))))))

(set! net_dutch_frans_lpc_group 
      (list
       '(name "frans_lpc_group")
       (list 'index_file 
	     (path-append net_dutch_frans_dir "group/franslpc.group"))
       '(grouped "true")
       (list 'default_diphone 
	     (string-append
	      (car (cadr (car (PhoneSet.description '(silences)))))
	      "-"
	      (car (cadr (car (PhoneSet.description '(silences)))))))))

;; Go ahead and set up the diphone db
(set! net_dutch_frans_db_name (us_diphone_init net_dutch_frans_lpc_sep))
;; Once you've built the group file you can comment out the above and
;; uncomment the following.
;(set! net_dutch_frans_db_name (us_diphone_init net_dutch_frans_lpc_group))

(define (net_dutch_frans_diphone_fix utt)
"(net_dutch_frans_diphone_fix UTT)
Map phones to phonological variants if the diphone database supports
them."
  (mapcar
   (lambda (s)
     (let ((name (item.name s)))
       (net_dutch_frans_diphone_fix_phone_name utt s)
       ))
   (utt.relation.items utt 'Segment))
  utt)

(define (net_dutch_frans_diphone_fix_phone_name utt seg)
"(net_dutch_frans_fix_phone_name UTT SEG)
Add the feature diphone_phone_name to given segment with the appropriate
name for constructing a diphone.  Basically adds _ if either side is part
of the same consonant cluster, adds $ either side if in different
syllable for preceding/succeeding vowel syllable."
  (let ((name (item.name seg)))
    (cond
     ((string-equal name "pau") t)
     ((string-equal "-" (item.feat seg 'ph_vc))
      (if (and (member_string name '(r w y l))
	       (member_string (item.feat seg "p.name") '(p t k b d g))
	       (item.relation.prev seg "SylStructure"))
	  (item.set_feat seg "us_diphone_right" (format nil "_%s" name)))
      (if (and (member_string name '(w y l m n p t k))
	       (string-equal (item.feat seg "p.name") 's)
	       (item.relation.prev seg "SylStructure"))
	  (item.set_feat seg "us_diphone_right" (format nil "_%s" name)))
      (if (and (string-equal name 's)
	       (member_string (item.feat seg "n.name") '(w y l m n p t k))
	       (item.relation.next seg "SylStructure"))
	  (item.set_feat seg "us_diphone_left" (format nil "%s_" name)))
      (if (and (string-equal name 'hh)
	       (string-equal (item.feat seg "n.name") 'y))
	  (item.set_feat seg "us_diphone_left" (format nil "%s_" name)))
      (if (and (string-equal name 'y)
	       (string-equal (item.feat seg "p.name") 'hh))
	  (item.set_feat seg "us_diphone_right" (format nil "_%s" name)))
      (if (and (member_string name '(p t k b d g))
	       (member_string (item.feat seg "n.name") '(r w y l))
	       (item.relation.next seg "SylStructure"))
	  (item.set_feat seg "us_diphone_left" (format nil "%s_" name)))
      )
     ((string-equal "ah" (item.name seg))
      (item.set_feat seg "us_diphone" "aa"))

   )))

(define (net_dutch_frans_voice_reset)
  "(net_dutch_frans_voice_reset)
Reset global variables back to previous voice."
  (net_dutch_frans::reset_phoneset)
  (net_dutch_frans::reset_tokenizer)
  (net_dutch_frans::reset_tagger)
  (net_dutch_frans::reset_lexicon)
  (net_dutch_frans::reset_phrasing)
  (net_dutch_frans::reset_intonation)
  (net_dutch_frans::reset_duration)
  (net_dutch_frans::reset_f0model)
  (net_dutch_frans::reset_other)
)

;;;  Full voice definition 
(define (voice_net_dutch_frans_diphone)
"(voice_net_dutch_frans_diphone)
Set speaker to frans in us from net."
  ;; Select appropriate phone set
  (net_dutch_frans::select_phoneset)

  ;; Select appropriate tokenization
  (net_dutch_frans::select_tokenizer)

  ;; For part of speech tagging
  (net_dutch_frans::select_tagger)

  (net_dutch_frans::select_lexicon)

  (net_dutch_frans::select_phrasing)

  (net_dutch_frans::select_intonation)

  (net_dutch_frans::select_duration)

  (net_dutch_frans::select_f0model)

  ;; Waveform synthesizer: UniSyn diphones
  (set! UniSyn_module_hooks (list net_dutch_frans_diphone_fix))
  (set! us_abs_offset 0.0)
  (set! window_factor 1.0)
  (set! us_rel_offset 0.0)
  (set! us_gain 0.9)

  (Parameter.set 'Synth_Method 'UniSyn)
  (Parameter.set 'us_sigpr 'lpc)
  (us_db_select net_dutch_frans_db_name)

  ;; This is where you can modify power (and sampling rate) if desired
  (set! after_synth_hooks nil)
;  (set! after_synth_hooks
;      (list
;        (lambda (utt)
;          (utt.wave.rescale utt 2.1))))

  ;; set callback to restore some original values changed by this voice
  (set! current_voice_reset net_dutch_frans_voice_reset)

  (set! current-voice 'net_dutch_frans_diphone)
)

(proclaim_voice
 'net_dutch_frans_diphone
 '((language english)
   (gender COMMENT)
   (dialect american)
   (description
    "COMMENT"
    )
   (builtwith festvox-1.3)))

(provide 'net_dutch_frans_diphone)
